@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2005-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,7 +33,6 @@
 #include <libempathy/empathy-debug.h>
 
 #include "empathy-contact-list-store.h"
-#include "empathy-contact-groups.h"
 #include "empathy-ui-utils.h"
 #include "empathy-gtk-enum-types.h"
 
@@ -43,26 +42,24 @@
  * (e.g. online, offline or from normal to a busy state).
  */
 
-/* Time user is shown as active */
-#define ACTIVE_USER_SHOW_TIME 7000
+/* Time in seconds user is shown as active */
+#define ACTIVE_USER_SHOW_TIME 7
 
-/* Time after connecting which we wait before active users are enabled */
-#define ACTIVE_USER_WAIT_TO_ENABLE_TIME 5000
+/* Time in seconds after connecting which we wait before active users are enabled */
+#define ACTIVE_USER_WAIT_TO_ENABLE_TIME 5
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_CONTACT_LIST_STORE, EmpathyContactListStorePriv))
 
-struct _EmpathyContactListStorePriv {
+typedef struct {
 	EmpathyContactList         *list;
 	gboolean                    show_offline;
 	gboolean                    show_avatars;
+	gboolean                    show_groups;
 	gboolean                    is_compact;
 	gboolean                    show_active;
 	EmpathyContactListStoreSort sort_criterium;
 	guint                       inhibit_active;
-
-	EmpathyContactGroupsFunc    get_contact_groups;
-	gpointer                    get_contact_groups_data;
-};
+} EmpathyContactListStorePriv;
 
 typedef struct {
 	GtkTreeIter  iter;
@@ -159,14 +156,63 @@ static gboolean         contact_list_store_update_list_mode_foreach  (GtkTreeMod
 
 enum {
 	PROP_0,
+	PROP_CONTACT_LIST,
 	PROP_SHOW_OFFLINE,
 	PROP_SHOW_AVATARS,
+	PROP_SHOW_GROUPS,
 	PROP_IS_COMPACT,
 	PROP_SORT_CRITERIUM
 };
 
 G_DEFINE_TYPE (EmpathyContactListStore, empathy_contact_list_store, GTK_TYPE_TREE_STORE);
 
+
+static gboolean
+contact_list_store_iface_setup (gpointer user_data)
+{
+	EmpathyContactListStore     *store = user_data;
+	EmpathyContactListStorePriv *priv = GET_PRIV (store);
+	GList                       *contacts, *l;
+
+	/* Signal connection. */
+	g_signal_connect (priv->list,
+			  "members-changed",
+			  G_CALLBACK (contact_list_store_members_changed_cb),
+			  store);
+	g_signal_connect (priv->list,
+			  "groups-changed",
+			  G_CALLBACK (contact_list_store_groups_changed_cb),
+			  store);
+
+	/* Add contacts already created. */
+	contacts = empathy_contact_list_get_members (priv->list);
+	for (l = contacts; l; l = l->next) {
+		contact_list_store_members_changed_cb (priv->list, l->data,
+						       NULL, 0, NULL,
+						       TRUE,
+						       store);
+
+		g_object_unref (l->data);
+	}
+	g_list_free (contacts);
+
+	return FALSE;
+}
+
+
+static void
+contact_list_store_set_contact_list (EmpathyContactListStore *store,
+				     EmpathyContactList      *list_iface)
+{
+	EmpathyContactListStorePriv *priv = GET_PRIV (store);
+
+	priv->list = g_object_ref (list_iface);
+
+	/* Let a chance to have all properties set before populating */
+	g_idle_add (contact_list_store_iface_setup,
+		    store);
+}
+
 static void
 empathy_contact_list_store_class_init (EmpathyContactListStoreClass *klass)
 {
@@ -176,6 +222,14 @@ empathy_contact_list_store_class_init (EmpathyContactListStoreClass *klass)
 	object_class->get_property = contact_list_store_get_property;
 	object_class->set_property = contact_list_store_set_property;
 
+	g_object_class_install_property (object_class,
+					 PROP_CONTACT_LIST,
+					 g_param_spec_object ("contact-list",
+							      "The contact list iface",
+							      "The contact list iface",
+							      EMPATHY_TYPE_CONTACT_LIST,
+							      G_PARAM_CONSTRUCT_ONLY |
+							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_OFFLINE,
 					 g_param_spec_boolean ("show-offline",
@@ -192,6 +246,14 @@ empathy_contact_list_store_class_init (EmpathyContactListStoreClass *klass)
 								"avatars for contacts",
 								TRUE,
 								G_PARAM_READWRITE));
+	 g_object_class_install_property (object_class,
+					  PROP_SHOW_GROUPS,
+					  g_param_spec_boolean ("show-groups",
+								"Show Groups",
+								"Whether contact list should display "
+								"contact groups",
+								TRUE,
+								G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_IS_COMPACT,
 					 g_param_spec_boolean ("is-compact",
@@ -220,9 +282,11 @@ empathy_contact_list_store_init (EmpathyContactListStore *store)
 	priv = GET_PRIV (store);
 
 	priv->show_avatars = TRUE;
-	priv->inhibit_active = g_timeout_add (ACTIVE_USER_WAIT_TO_ENABLE_TIME,
-					      (GSourceFunc) contact_list_store_inibit_active_cb,
-					      store);
+	priv->show_groups = TRUE;
+	priv->inhibit_active = g_timeout_add_seconds (ACTIVE_USER_WAIT_TO_ENABLE_TIME,
+						      (GSourceFunc) contact_list_store_inibit_active_cb,
+						      store);
+	contact_list_store_setup (store);
 }
 
 static void
@@ -264,12 +328,18 @@ contact_list_store_get_property (GObject    *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
+	case PROP_CONTACT_LIST:
+		g_value_set_object (value, priv->list);
+		break;
 	case PROP_SHOW_OFFLINE:
 		g_value_set_boolean (value, priv->show_offline);
 		break;
 	case PROP_SHOW_AVATARS:
 		g_value_set_boolean (value, priv->show_avatars);
 		break;
+	case PROP_SHOW_GROUPS:
+		g_value_set_boolean (value, priv->show_groups);
+		break;
 	case PROP_IS_COMPACT:
 		g_value_set_boolean (value, priv->is_compact);
 		break;
@@ -293,6 +363,10 @@ contact_list_store_set_property (GObject      *object,
 	priv = GET_PRIV (object);
 
 	switch (param_id) {
+	case PROP_CONTACT_LIST:
+		contact_list_store_set_contact_list (EMPATHY_CONTACT_LIST_STORE (object),
+						     g_value_get_object (value));
+		break;
 	case PROP_SHOW_OFFLINE:
 		empathy_contact_list_store_set_show_offline (EMPATHY_CONTACT_LIST_STORE (object),
 							    g_value_get_boolean (value));
@@ -301,6 +375,10 @@ contact_list_store_set_property (GObject      *object,
 		empathy_contact_list_store_set_show_avatars (EMPATHY_CONTACT_LIST_STORE (object),
 							    g_value_get_boolean (value));
 		break;
+	case PROP_SHOW_GROUPS:
+		empathy_contact_list_store_set_show_groups (EMPATHY_CONTACT_LIST_STORE (object),
+							    g_value_get_boolean (value));
+		break;
 	case PROP_IS_COMPACT:
 		empathy_contact_list_store_set_is_compact (EMPATHY_CONTACT_LIST_STORE (object),
 							  g_value_get_boolean (value));
@@ -318,41 +396,11 @@ contact_list_store_set_property (GObject      *object,
 EmpathyContactListStore *
 empathy_contact_list_store_new (EmpathyContactList *list_iface)
 {
-	EmpathyContactListStore     *store;
-	EmpathyContactListStorePriv *priv;
-	GList                      *contacts, *l;
-
 	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST (list_iface), NULL);
 
-	store = g_object_new (EMPATHY_TYPE_CONTACT_LIST_STORE, NULL);
-	priv = GET_PRIV (store);
-
-	contact_list_store_setup (store);
-	priv->list = g_object_ref (list_iface);
-
-	/* Signal connection. */
-	g_signal_connect (priv->list,
-			  "members-changed",
-			  G_CALLBACK (contact_list_store_members_changed_cb),
-			  store);
-	g_signal_connect (priv->list,
-			  "groups-changed",
-			  G_CALLBACK (contact_list_store_groups_changed_cb),
-			  store);
-
-	/* Add contacts already created. */
-	contacts = empathy_contact_list_get_members (priv->list);
-	for (l = contacts; l; l = l->next) {
-		contact_list_store_members_changed_cb (priv->list, l->data,
-						       NULL, 0, NULL,
-						       TRUE,
-						       store);
-
-		g_object_unref (l->data);
-	}
-	g_list_free (contacts);
-
-	return store;
+	return g_object_new (EMPATHY_TYPE_CONTACT_LIST_STORE,
+			     "contact-list", list_iface,
+			     NULL);
 }
 
 EmpathyContactList *
@@ -407,6 +455,8 @@ empathy_contact_list_store_set_show_offline (EmpathyContactListStore *store,
 
 	/* Restore to original setting. */
 	priv->show_active = show_active;
+
+	g_object_notify (G_OBJECT (store), "show-offline");
 }
 
 gboolean
@@ -440,6 +490,54 @@ empathy_contact_list_store_set_show_avatars (EmpathyContactListStore *store,
 				(GtkTreeModelForeachFunc)
 				contact_list_store_update_list_mode_foreach,
 				store);
+
+	g_object_notify (G_OBJECT (store), "show-avatars");
+}
+
+gboolean
+empathy_contact_list_store_get_show_groups (EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store), TRUE);
+
+	priv = GET_PRIV (store);
+
+	return priv->show_groups;
+}
+
+void
+empathy_contact_list_store_set_show_groups (EmpathyContactListStore *store,
+					    gboolean                 show_groups)
+{
+	EmpathyContactListStorePriv *priv;
+	GList                       *contacts, *l;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store));
+
+	priv = GET_PRIV (store);
+
+	if (priv->show_groups == show_groups) {
+		return;
+	}
+
+	priv->show_groups = show_groups;
+
+	/* Remove all contacts and add them back, not optimized but that's the
+	 * easy way :) */
+	gtk_tree_store_clear (GTK_TREE_STORE (store));
+	contacts = empathy_contact_list_get_members (priv->list);
+	for (l = contacts; l; l = l->next) {
+		contact_list_store_members_changed_cb (priv->list, l->data,
+						       NULL, 0, NULL,
+						       TRUE,
+						       store);
+
+		g_object_unref (l->data);
+	}
+	g_list_free (contacts);
+
+	g_object_notify (G_OBJECT (store), "show-groups");
 }
 
 gboolean
@@ -473,6 +571,8 @@ empathy_contact_list_store_set_is_compact (EmpathyContactListStore *store,
 				(GtkTreeModelForeachFunc)
 				contact_list_store_update_list_mode_foreach,
 				store);
+
+	g_object_notify (G_OBJECT (store), "is-compact");
 }
 
 EmpathyContactListStoreSort
@@ -502,16 +602,18 @@ empathy_contact_list_store_set_sort_criterium (EmpathyContactListStore     *stor
 	switch (sort_criterium) {
 	case EMPATHY_CONTACT_LIST_STORE_SORT_STATE:
 		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
-						      COL_STATUS,
+						      EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
 						      GTK_SORT_ASCENDING);
 		break;
 		
 	case EMPATHY_CONTACT_LIST_STORE_SORT_NAME:
 		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
-						      COL_NAME,
+						      EMPATHY_CONTACT_LIST_STORE_COL_NAME,
 						      GTK_SORT_ASCENDING);
 		break;
 	}
+
+	g_object_notify (G_OBJECT (store), "sort-criterium");
 }
 
 gboolean
@@ -524,7 +626,7 @@ empathy_contact_list_store_row_separator_func (GtkTreeModel *model,
 	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_SEPARATOR, &is_separator,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator,
 			    -1);
 
 	return is_separator;
@@ -550,8 +652,8 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 	}
 
 	gtk_tree_model_get (model, &iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	if (!is_group) {
@@ -565,8 +667,8 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 		iter = parent_iter;
 
 		gtk_tree_model_get (model, &iter,
-				    COL_IS_GROUP, &is_group,
-				    COL_NAME, &name,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 				    -1);
 		if (!is_group) {
 			g_free (name);
@@ -598,7 +700,9 @@ empathy_contact_list_store_search_equal_func (GtkTreeModel *model,
 		return TRUE;
 	}
 
-	gtk_tree_model_get (model, iter, COL_NAME, &name, -1);
+	gtk_tree_model_get (model, iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
+			    -1);
 
 	if (!name) {
 		return TRUE;
@@ -631,7 +735,7 @@ contact_list_store_finalize_foreach (GtkTreeModel *model,
 	EmpathyContact          *contact = NULL;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
-			    COL_CONTACT, &contact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
 			    -1);
 
 	if (contact) {
@@ -662,15 +766,17 @@ contact_list_store_setup (EmpathyContactListStore *store)
 	
 	priv = GET_PRIV (store);
 
-	gtk_tree_store_set_column_types (GTK_TREE_STORE (store), COL_COUNT, types);
+	gtk_tree_store_set_column_types (GTK_TREE_STORE (store),
+					 EMPATHY_CONTACT_LIST_STORE_COL_COUNT,
+					 types);
 
 	/* Set up sorting */
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (store),
-					 COL_NAME,
+					 EMPATHY_CONTACT_LIST_STORE_COL_NAME,
 					 contact_list_store_name_sort_func,
 					 store, NULL);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (store),
-					 COL_STATUS,
+					 EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
 					 contact_list_store_state_sort_func,
 					 store, NULL);
 
@@ -714,6 +820,9 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 		g_signal_connect (contact, "notify::presence",
 				  G_CALLBACK (contact_list_store_contact_updated_cb),
 				  store);
+		g_signal_connect (contact, "notify::presence-message",
+				  G_CALLBACK (contact_list_store_contact_updated_cb),
+				  store);
 		g_signal_connect (contact, "notify::name",
 				  G_CALLBACK (contact_list_store_contact_updated_cb),
 				  store);
@@ -767,7 +876,7 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 {
 	EmpathyContactListStorePriv *priv;
 	GtkTreeIter                 iter;
-	GList                      *groups, *l;
+	GList                      *groups = NULL, *l;
 
 	priv = GET_PRIV (store);
 	
@@ -775,17 +884,19 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		return;
 	}
 
-	groups = empathy_contact_list_get_groups (priv->list, contact);
+	if (priv->show_groups) {
+		groups = empathy_contact_list_get_groups (priv->list, contact);
+	}
 
 	/* If no groups just add it at the top level. */
 	if (!groups) {
 		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    COL_NAME, empathy_contact_get_name (contact),
-				    COL_CONTACT, contact,
-				    COL_IS_GROUP, FALSE,
-				    COL_IS_SEPARATOR, FALSE,
-				    COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
 				    -1);
 	}
 
@@ -798,11 +909,11 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
 					     &iter_group, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    COL_NAME, empathy_contact_get_name (contact),
-				    COL_CONTACT, contact,
-				    COL_IS_GROUP, FALSE,
-				    COL_IS_SEPARATOR, FALSE,
-				    COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
 				    -1);
 		g_free (l->data);
 	}
@@ -934,7 +1045,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 		/* Get online state before. */
 		if (iters && g_list_length (iters) > 0) {
 			gtk_tree_model_get (model, iters->data,
-					    COL_IS_ONLINE, &was_online,
+					    EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE, &was_online,
 					    -1);
 		}
 
@@ -962,16 +1073,16 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	pixbuf_avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
 	for (l = iters; l && set_model; l = l->next) {
 		gtk_tree_store_set (GTK_TREE_STORE (store), l->data,
-				    COL_ICON_STATUS, empathy_icon_name_for_contact (contact),
-				    COL_PIXBUF_AVATAR, pixbuf_avatar,
-				    COL_PIXBUF_AVATAR_VISIBLE, priv->show_avatars,
-				    COL_NAME, empathy_contact_get_name (contact),
-				    COL_STATUS, empathy_contact_get_status (contact),
-				    COL_STATUS_VISIBLE, !priv->is_compact,
-				    COL_IS_GROUP, FALSE,
-				    COL_IS_ONLINE, now_online,
-				    COL_IS_SEPARATOR, FALSE,
-				    COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, empathy_icon_name_for_contact (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, pixbuf_avatar,
+				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, priv->show_avatars,
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS, empathy_contact_get_status (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE, now_online,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
 				    -1);
 	}
 
@@ -984,9 +1095,9 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 
 		if (do_set_active) {
 			data = contact_list_store_contact_active_new (store, contact, do_remove);
-			g_timeout_add (ACTIVE_USER_SHOW_TIME,
-				       (GSourceFunc) contact_list_store_contact_active_cb,
-				       data);
+			g_timeout_add_seconds (ACTIVE_USER_SHOW_TIME,
+					       (GSourceFunc) contact_list_store_contact_active_cb,
+					       data);
 		}
 	}
 
@@ -1029,7 +1140,7 @@ contact_list_store_contact_set_active (EmpathyContactListStore *store,
 		GtkTreePath *path;
 
 		gtk_tree_store_set (GTK_TREE_STORE (store), l->data,
-				    COL_IS_ACTIVE, active,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, active,
 				    -1);
 
 		empathy_debug (DEBUG_DOMAIN, "Set item %s", active ? "active" : "inactive");
@@ -1121,8 +1232,8 @@ contact_list_store_get_group_foreach (GtkTreeModel *model,
 	}
 
 	gtk_tree_model_get (model, iter,
-			    COL_NAME, &str,
-			    COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &str,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 			    -1);
 
 	if (is_group && strcmp (str, fg->name) == 0) {
@@ -1166,11 +1277,11 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 
 		gtk_tree_store_append (GTK_TREE_STORE (store), &iter_group, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_group,
-				    COL_ICON_STATUS, NULL,
-				    COL_NAME, name,
-				    COL_IS_GROUP, TRUE,
-				    COL_IS_ACTIVE, FALSE,
-				    COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, NULL,
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, name,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, TRUE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
 				    -1);
 
 		if (iter_group_to_set) {
@@ -1181,7 +1292,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 				       &iter_separator, 
 				       &iter_group);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_separator,
-				    COL_IS_SEPARATOR, TRUE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, TRUE,
 				    -1);
 
 		if (iter_separator_to_set) {
@@ -1202,7 +1313,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			gboolean is_separator;
 
 			gtk_tree_model_get (model, &iter_separator,
-					    COL_IS_SEPARATOR, &is_separator,
+					    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator,
 					    -1);
 
 			if (is_separator && iter_separator_to_set) {
@@ -1212,6 +1323,28 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 	}
 }
 
+static guint
+contact_list_store_ordered_presence (McPresence state)
+{
+	switch (state) {
+	case MC_PRESENCE_UNSET:
+	case MC_PRESENCE_OFFLINE:
+		return 5;
+	case MC_PRESENCE_AVAILABLE:
+		return 0;
+	case MC_PRESENCE_AWAY:
+		return 2;
+	case MC_PRESENCE_EXTENDED_AWAY:
+		return 3;
+	case MC_PRESENCE_HIDDEN:
+		return 4;
+	case MC_PRESENCE_DO_NOT_DISTURB:
+		return 1;
+	default:
+		g_return_val_if_reached (6);
+	}
+}
+
 static gint
 contact_list_store_state_sort_func (GtkTreeModel *model,
 				    GtkTreeIter  *iter_a,
@@ -1221,19 +1354,18 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 	gint            ret_val = 0;
 	gchar          *name_a, *name_b;
 	gboolean        is_separator_a, is_separator_b;
-	EmpathyContact  *contact_a, *contact_b;
-	EmpathyPresence *presence_a, *presence_b;
-	McPresence      state_a, state_b;
+	EmpathyContact *contact_a, *contact_b;
+	guint           presence_a, presence_b;
 
 	gtk_tree_model_get (model, iter_a,
-			    COL_NAME, &name_a,
-			    COL_CONTACT, &contact_a,
-			    COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
-			    COL_NAME, &name_b,
-			    COL_CONTACT, &contact_b,
-			    COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
 			    -1);
 
 	/* Separator or group? */
@@ -1260,27 +1392,17 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 	 * the presences.
 	 */
 	presence_a = empathy_contact_get_presence (EMPATHY_CONTACT (contact_a));
+	presence_a = contact_list_store_ordered_presence (presence_a);
 	presence_b = empathy_contact_get_presence (EMPATHY_CONTACT (contact_b));
+	presence_b = contact_list_store_ordered_presence (presence_b);
 
-	if (!presence_a && presence_b) {
-		ret_val = 1;
-	} else if (presence_a && !presence_b) {
+	if (presence_a < presence_b) {
 		ret_val = -1;
-	} else if (!presence_a && !presence_b) {
-		/* Both offline, sort by name */
-		ret_val = g_utf8_collate (name_a, name_b);
+	} else if (presence_a > presence_b) {
+		ret_val = 1;
 	} else {
-		state_a = empathy_presence_get_state (presence_a);
-		state_b = empathy_presence_get_state (presence_b);
-
-		if (state_a < state_b) {
-			ret_val = -1;
-		} else if (state_a > state_b) {
-			ret_val = 1;
-		} else {
-			/* Fallback: compare by name */
-			ret_val = g_utf8_collate (name_a, name_b);
-		}
+		/* Fallback: compare by name */
+		ret_val = g_utf8_collate (name_a, name_b);
 	}
 
 free_and_out:
@@ -1310,14 +1432,14 @@ contact_list_store_name_sort_func (GtkTreeModel *model,
 	gint           ret_val;
 
 	gtk_tree_model_get (model, iter_a,
-			    COL_NAME, &name_a,
-			    COL_CONTACT, &contact_a,
-			    COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
-			    COL_NAME, &name_b,
-			    COL_CONTACT, &contact_b,
-			    COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
 			    -1);
 
 	/* If contact is NULL it means it's a group. */
@@ -1359,7 +1481,7 @@ contact_list_store_find_contact_foreach (GtkTreeModel *model,
 	EmpathyContact *contact;
 
 	gtk_tree_model_get (model, iter,
-			    COL_CONTACT, &contact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
 			    -1);
 
 	if (!contact) {
@@ -1418,8 +1540,8 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	}
 
 	gtk_tree_store_set (GTK_TREE_STORE (store), iter,
-			    COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
-			    COL_STATUS_VISIBLE, !priv->is_compact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
+			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
 			    -1);
 
 	return FALSE;
