@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2004-2007 Imendio AB
- * Copyright (C) 2007 Collabora Ltd.
+ * Copyright (C) 2007-2008 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@ struct _EmpathyMessagePriv {
 	EmpathyContact     *sender;
 	EmpathyContact     *receiver;
 	gchar              *body;
-	EmpathyTime         timestamp;
+	time_t              timestamp;
 };
 
 static void empathy_message_class_init (EmpathyMessageClass *class);
@@ -385,7 +385,7 @@ empathy_message_set_body (EmpathyMessage *message,
 	g_object_notify (G_OBJECT (message), "body");
 }
 
-EmpathyTime
+time_t
 empathy_message_get_timestamp (EmpathyMessage *message)
 {
 	EmpathyMessagePriv *priv;
@@ -399,7 +399,7 @@ empathy_message_get_timestamp (EmpathyMessage *message)
 
 void
 empathy_message_set_timestamp (EmpathyMessage *message,
-			      EmpathyTime     timestamp)
+			       time_t          timestamp)
 {
 	EmpathyMessagePriv *priv;
 
@@ -417,6 +417,26 @@ empathy_message_set_timestamp (EmpathyMessage *message,
 	g_object_notify (G_OBJECT (message), "timestamp");
 }
 
+GDate *
+empathy_message_get_date_and_time (EmpathyMessage *message, time_t *timestamp)
+{
+	GDate *date;
+
+	*timestamp = 0;
+	if (message) {
+		*timestamp = empathy_message_get_timestamp (message);
+	}
+
+	if (timestamp <= 0) {
+		*timestamp = empathy_time_get_current ();
+	}
+
+	date = g_date_new ();
+	g_date_set_time (date, *timestamp);
+
+	return date;
+}
+
 EmpathyMessageType
 empathy_message_type_from_str (const gchar *type_str)
 {
